
// source:
// http://en.cppreference.com/w/cpp/algorithm/inner_product

#include <numeric>
#include <iostream>
#include <vector>
#include <functional>
#include <iterator>

int main()
{
    std::vector<int> a{0, 1, 2, 3, 4};
    std::vector<int> b{5, 4, 2, 3, 1};
    std::cout << "a={ ";
    for(auto x: a) std::cout << x << ", ";
    std::cout << "}\n";

    std::cout << "b={ ";
    for(auto x: b) std::cout << x << ", ";
    std::cout << "}\n";

    int r1 = std::inner_product(begin(a), end(a), begin(b), 0);
    std::cout << "Inner product of a and b: " << r1 << '\n';

    int r2 = std::inner_product(a.begin(), a.end(), b.begin(), 0,
                                std::plus<int>(), std::equal_to<int>());
    std::cout << "Number of pairwise matches between a and b: " <<  r2 << '\n';

    int r3 = std::inner_product(a.begin(), a.end(), b.begin(), 0,
                                [](int a, int b) { return a+b; },
                                [](int a, int b)->int { return a!=b; });
    std::cout << "Number of pairwise mismatches between a and b: " <<  r3 << '\n';

    int product = std::accumulate(begin(b), end(b), 1, std::multiplies<int>());
    std::cout << "accumulate/product b: " <<  product << '\n';
    std::string s = std::accumulate(std::begin(b), std::end(b), std::string{},
                                    [](const std::string& a, int b) {
                                        return a.empty() ? std::to_string(b)
                                               : a + '-' + std::to_string(b);
                                    });
    std::cout << "special accumulate b: " <<  s << '\n';

    std::cout << "partial_sum of a: ";
    std::partial_sum(begin(a), end(a), std::ostream_iterator<int>(std::cout, " "));
    std::cout << '\n';
}
