
// concepts/requires
// source:cppreference.com

#if 1

// concept definition
template<typename T>
concept Addable = requires (T x) { x + x; }; // requires-expression

// concept usage
template<typename T> requires Addable<T> // requires-clause, not requires-expression
T add(T a, T b) { return a + b; }

// better, more readable variant (constrained C++20 function template)
template<Addable T>
T add2(T a, T b) { return a + b; }

#else

template<typename T>
    requires requires (T x) { x + x; } // ad-hoc constraint, note keyword used twice
T add(T a, T b) { return a + b; }

#endif

int main() {
    return add(1,2);
}
