
// C++20: class operator <=> created by default
// source: cppreference.com

#include <iostream>
#include <set>
#include <compare>

struct Point {
  int x;
  int y;
  auto operator<=>(const Point&) const = default;
  // compiler generates all six relational operators

  // ...
};
std::ostream & operator << (std::ostream &o, Point p) {
    return o << "Point{" << p.x << "," << p.y << "}";
}

#define PRINT(cmp) \
  std::cout << #cmp "\t " << std::boolalpha << (cmp) << "\n"

int main() {
  Point pt1{1, 1};
  Point pt2{1, 2};
  PRINT(pt1);
  PRINT(pt2);
  std::set<Point> s;    // set needs operator <
  s.insert(pt1);        // O.K.
  PRINT(pt1 == pt2);     // false; operator== is implicitly defaulted.
  PRINT(pt1 != pt2);     // true
  PRINT(pt1 <  pt2);     // true (lexicographic order)
  PRINT(pt1 <= pt2);     // true
  PRINT(pt1 >  pt2);     // false
  PRINT(pt1 >= pt2);     // false
}

