
// C++17 std::optional example

#include <string>
#include <functional>
#include <iostream>
#include <optional>

// optional can be used as the return type of a factory that may fail
std::optional<std::string> create1(bool b) {
    if (b)
        return "C1";
    return {};
}

// std::nullopt represents any empty std::optional
auto create2(bool b) {
    return b ? std::optional<std::string>{"C2"} : std::nullopt;
}

// std::reference_wrapper may be used to return a reference
auto create_ref(bool b) {
    static std::string value = "Alpha";
    return b ? std::optional<std::reference_wrapper<std::string>>{value}
             : std::nullopt;
}

int main()
{
    std::cout << "create1(false) returned "
              << create1(false).value_or("empty") << '\n';

    // optional-returning factory functions are usable as conditions of while
    // and if
    if (auto str = create2(true)) {
        std::cout << "create2(true) returned " << *str << '\n';
    }

    if (auto str = create_ref(true)) {
        // using get() to access the reference_wrapper's value
        std::cout << "create_ref(true) returned " << str->get() << '\n';
        str->get() = "Beta";
        std::cout << "modifying it changed it to " << str->get() << '\n';
    }
    if (auto str = create_ref(true)) {
        std::cout << "Next create_ref(true) returned " << str->get() << '\n';
    }
}

