
// source:
// https://en.cppreference.com/w/cpp/utility/functional/reference_wrapper

#include <algorithm>
#include <list>
#include <vector>
#include <iostream>
#include <numeric>
#include <random>
#include <functional>

// template for printing
void print(auto & c) {
    for (int x : c){
        std::cout << x << ' ';
    }
    std::cout << '\n';
}

int main()
{
    std::list<int> l(7);
    std::cout << "Fill list: ";
    std::iota(begin(l), end(l), -3);
    print(l);

    std::cout << "Make vector<std::reference_wrapper<int>> to list ...\n";
    std::vector<std::reference_wrapper<int>> v(begin(l), end(l));

    std::cout << "Shuffle the vector<std::reference_wrapper<int>> ... \n";
    // can't use shuffle on a list (requires random access), but can use it on
    // a vector
    std::shuffle(begin(v), end(v), std::mt19937{std::random_device{}()});

    std::cout << "Contents of the list: ";
    print(l);
    std::cout << "Contents of the list, as seen through a shuffled vector: ";
    print(v);
    std::cout << "Doubling the values in the initial list...\n";
    for (int& i : l) {
        i *= 2;
    }
    std::cout << "Contents of the list, as seen through a shuffled vector: ";
    print(v);
}

