
#include <variant>
#include <string>
#include <cassert>
#include <iostream>

using namespace std::literals;  // needed for "..."s

int main()
{
    std::variant<int, float> v, w;
    v = 12;             // v contains int
    int i = std::get<int>(v);
    std::cout << "i = " << i << "\n";
    w = std::get<int>(v);
    w = std::get<0>(v); // same effect as the previous line
    w = v;              // same effect as the previous line
    // std::cout << "w = " << w << "\n"; // not defined?

    // compile-time errors:
//  std::get<double>(v); // error: no double in v
//  std::get<3>(v);      // error: valid index values are 0 and 1

    // run-time error:
    try {
        std::get<float>(w); // w contains int, not float: will throw
    }
    catch (const std::bad_variant_access&) {
        std::cout << "Exception: bad_variant_access\n";
    }

    std::variant<std::string> x("abc"); // converting constructors work when unambiguous
    x = "def"; // converting assignment also works when unambiguous

    // ambiguity:
    std::variant<std::string, bool> y("abc"); // ? casts to bool when passed a char const *
#if 0
    assert(std::holds_alternative<bool>(y));  // succeeds ? TODO:
#endif
    if(std::holds_alternative<bool>(y))
        std::cout << "y contains bool variant\n";
    y = "xyz"s;
    assert(std::holds_alternative<std::string>(y)); // O.K.
    if(std::holds_alternative<std::string>(y))
        std::cout << "y contains string variant\n";
}

