
// source: http://www.cplusplus.com/reference/
// numpunct::grouping example

#include <iostream>       // std::cout
#include <string>         // std::string
#include <locale>         // std::locale, std::numpunct, std::use_facet
#include <stdexcept>
#include <iterator>     // begin(), end()

/*
   The sequence is returned as a string (basic_string<char>) where each char
   element shall be interpreted as an integer value indicating the number of
   digits in each group. Grouping is considered from right to left, with the
   first character in the string being used for the rightmost group, and the last
   character used for any group farther away than the length of the string.

   For example, "\03" indicates groups of three digits each, while "\02\03" would
   indicate that the rightmost group has two digits and all the remaining ones
   have three digits each.

   Internally, this function simply calls the virtual protected member
   do_grouping, which by default returns the value returned for the "C" locale:
   an empty string, indicating no grouping.
 */

// custom numpunct with grouping:
struct my_numpunct : std::numpunct<char> {
    std::string do_grouping() const {return "\03";}
};

int main() {
    std::locale loc (std::cout.getloc(),new my_numpunct);
    std::cout.imbue(loc);
    std::cout << "one million: " << 1000000 << '\n';
    // OUTPUT: one million: 1,000,000

    std::cout << "==================================================\n";

    try {
        std::locale foo("");     // locale depends on LC_* environment vars
        std::locale::global(foo);
        std::cout.imbue(foo);    // needed to make effect on cout
        std::cout << "The selected locale is: " << foo.name() << "\n\n";

        std::cout <<  "has_facet<numpunct> = ";
        std::cout << std::boolalpha << std::has_facet<std::numpunct<char>>(foo) << "\n";
        //TODO: std::has_facet<std::ctype<char>>(std::cout.getloc())
        std::cout <<  "has_facet<ctype> = ";
        std::cout << std::boolalpha << std::has_facet<std::ctype<char>>(foo) << "\n";

    }
    catch (const std::runtime_error &e) {
        std::cout << "exception:" << e.what() << '\n';
        return 1;
    }

    // numpunct::decimal_point example
    char point = std::use_facet< std::numpunct<char> >(std::cout.getloc()).decimal_point();
    std::cout << "decimal_point() = \'" << point << "\'\n";

    char str[] = "Some text";
    std::cout << "string = \"" << str << "\"\n";
    // ctyype::toupper example
    std::use_facet<std::ctype<char>>(std::cout.getloc()).toupper(std::begin(str),std::end(str));
    std::cout << "string = \"" << str << "\"\n";

}
