// cpp-hiding.cc
//
// hiding of inherited class members in C++
//

#include <iostream>             // cout
using namespace std;

/////////////////////////////////////////////////////////////////////////////
struct B {
    int i;
    B(): i(0) {}
    void f(int a) { cout << "B::f(" << a << ")\n"; }
    void f(double a) { cout << "B::f(" << a << ")\n"; }
    void f()      { cout << "B::f()\n"; }
};

struct D1 : B {                 // public inheritance
    int i;                      // hide B::i
    D1(): i(1) {}
    void f(int a, int b) {      // hide all B::f
        cout << "D1::f(" << a << ", " << b << ")\n"; 
    }
};

struct D2 : B {
    int i;                      // hide B::i
    D2(): i(2) {}
    void f(int a) {             // override B::f(int), hide other B::f
        cout << "D2::f(" << a << ")\n"; 
    }
};

struct D3 : B {
    int i;                      // hide B::i
    D3(): i(3) {}
    using B::f;                 // do not hide any B::f
    void f(int a) {             // override B::f(int)
        cout << "D3::f(" << a << ")\n"; 
    }
};

/////////////////////////////////////////////////////////////////////////////
void f(char) {
    cout << "void f(char)\n";
}

void g()
{
    extern void f(int);          // hide global ::f(char *)
    f('a');                      // ::f(int) selected
}

void f(int) {
    cout << "void f(int)\n";
}

/////////////////////////////////////////////////////////////////////////////
int main()
{
    B b;
    D1 d1;
    D2 d2;
    D3 d3;

    b.f(0);
    cout << "===== d1:\n";
    // d1.f(1); // hidden by  D1::f(int,int)
    d1.B::f(1);
    d1.f(1,1);
    cout << "===== d2:\n";
    d2.f(2);
    // d2.f();  // hidden by  D2::f(int)
    d2.f(2.71); // overload resolution uses D2::f(int) +WARNING
    cout << "===== d3:\n";
    d3.f(3);
    d3.f();     // NOT hidden
    d3.f(3.14); // NOT hidden


    cout << "===== \n";

    cout << "b.i="     << b.i << endl;
    cout << "d1.i="    << d1.i << "  ";
    cout << "d1.B::i=" << d1.B::i << endl;
    cout << "d2.i="    << d2.i << "  ";
    cout << "d2.B::i=" << d2.B::i << endl;

    g();
}

